@@ -1,22 +1,25 @@
-# -*- coding: utf-8 -*-
 """
 Module for formatting output data in HTML.
 """
 
-from __future__ import print_function
-
 from textwrap import dedent
+from typing import IO, Any, Dict, Iterable, List, Mapping, Optional, Tuple, Union, cast
 
-from pandas.compat import OrderedDict, lzip, map, range, u, unichr, zip
+from pandas._config import get_option
+
+from pandas._libs import lib
 
 from pandas.core.dtypes.generic import ABCMultiIndex
 
-from pandas import compat
-import pandas.core.common as com
-from pandas.core.config import get_option
+from pandas import option_context
 
-from pandas.io.common import _is_url
-from pandas.io.formats.format import TableFormatter, get_level_lengths
+from pandas.io.common import is_url
+from pandas.io.formats.format import (
+    DataFrameFormatter,
+    TableFormatter,
+    buffer_put_lines,
+    get_level_lengths,
+)
 from pandas.io.formats.printing import pprint_thing
 
 
@@ -32,32 +35,39 @@ class HTMLFormatter(TableFormatter):
 
     indent_delta = 2
 
-    def __init__(self, formatter, classes=None, border=None):
+    def __init__(
+        self,
+        formatter: DataFrameFormatter,
+        classes: Optional[Union[str, List[str], Tuple[str, ...]]] = None,
+        border: Optional[int] = None,
+    ) -> None:
         self.fmt = formatter
         self.classes = classes
 
         self.frame = self.fmt.frame
         self.columns = self.fmt.tr_frame.columns
-        self.elements = []
-        self.bold_rows = self.fmt.kwds.get('bold_rows', False)
-        self.escape = self.fmt.kwds.get('escape', True)
+        self.elements: List[str] = []
+        self.bold_rows = self.fmt.bold_rows
+        self.escape = self.fmt.escape
         self.show_dimensions = self.fmt.show_dimensions
         if border is None:
-            border = get_option('display.html.border')
+            border = cast(int, get_option("display.html.border"))
         self.border = border
         self.table_id = self.fmt.table_id
         self.render_links = self.fmt.render_links
+        if isinstance(self.fmt.col_space, int):
+            self.fmt.col_space = "{colspace}px".format(colspace=self.fmt.col_space)
 
     @property
-    def show_row_idx_names(self):
+    def show_row_idx_names(self) -> bool:
         return self.fmt.show_row_idx_names
 
     @property
-    def show_col_idx_names(self):
+    def show_col_idx_names(self) -> bool:
         return self.fmt.show_col_idx_names
 
     @property
-    def row_levels(self):
+    def row_levels(self) -> int:
         if self.fmt.index:
             # showing (row) index
             return self.frame.index.nlevels
@@ -71,100 +81,144 @@ class HTMLFormatter(TableFormatter):
         # not showing (row) index
         return 0
 
+    def _get_columns_formatted_values(self) -> Iterable:
+        return self.columns
+
+    # https://github.com/python/mypy/issues/1237
     @property
-    def is_truncated(self):
+    def is_truncated(self) -> bool:  # type: ignore
         return self.fmt.is_truncated
 
     @property
-    def ncols(self):
+    def ncols(self) -> int:
         return len(self.fmt.tr_frame.columns)
 
-    def write(self, s, indent=0):
+    def write(self, s: Any, indent: int = 0) -> None:
         rs = pprint_thing(s)
-        self.elements.append(' ' * indent + rs)
+        self.elements.append(" " * indent + rs)
 
-    def write_th(self, s, indent=0, tags=None):
-        if self.fmt.col_space is not None and self.fmt.col_space > 0:
-            tags = (tags or "")
-            tags += ('style="min-width: {colspace};"'
-                     .format(colspace=self.fmt.col_space))
+    def write_th(
+        self, s: Any, header: bool = False, indent: int = 0, tags: Optional[str] = None
+    ) -> None:
+        """
+        Method for writting a formatted <th> cell.
 
-        return self._write_cell(s, kind='th', indent=indent, tags=tags)
+        If col_space is set on the formatter then that is used for
+        the value of min-width.
 
-    def write_td(self, s, indent=0, tags=None):
-        return self._write_cell(s, kind='td', indent=indent, tags=tags)
+        Parameters
+        ----------
+        s : object
+            The data to be written inside the cell.
+        header : bool, default False
+            Set to True if the <th> is for use inside <thead>.  This will
+            cause min-width to be set if there is one.
+        indent : int, default 0
+            The indentation level of the cell.
+        tags : str, default None
+            Tags to include in the cell.
 
-    def _write_cell(self, s, kind='td', indent=0, tags=None):
+        Returns
+        -------
+        A written <th> cell.
+        """
+        if header and self.fmt.col_space is not None:
+            tags = tags or ""
+            tags += 'style="min-width: {colspace};"'.format(colspace=self.fmt.col_space)
+
+        self._write_cell(s, kind="th", indent=indent, tags=tags)
+
+    def write_td(self, s: Any, indent: int = 0, tags: Optional[str] = None) -> None:
+        self._write_cell(s, kind="td", indent=indent, tags=tags)
+
+    def _write_cell(
+        self, s: Any, kind: str = "td", indent: int = 0, tags: Optional[str] = None
+    ) -> None:
         if tags is not None:
-            start_tag = '<{kind} {tags}>'.format(kind=kind, tags=tags)
+            start_tag = "<{kind} {tags}>".format(kind=kind, tags=tags)
         else:
-            start_tag = '<{kind}>'.format(kind=kind)
+            start_tag = "<{kind}>".format(kind=kind)
 
         if self.escape:
             # escape & first to prevent double escaping of &
-            esc = OrderedDict([('&', r'&amp;'), ('<', r'&lt;'),
-                               ('>', r'&gt;')])
+            esc = {"&": r"&amp;", "<": r"&lt;", ">": r"&gt;"}
         else:
             esc = {}
 
         rs = pprint_thing(s, escape_chars=esc).strip()
 
-        if self.render_links and _is_url(rs):
+        if self.render_links and is_url(rs):
             rs_unescaped = pprint_thing(s, escape_chars={}).strip()
-            start_tag += '<a href="{url}" target="_blank">'.format(
-                url=rs_unescaped)
-            end_a = '</a>'
+            start_tag += '<a href="{url}" target="_blank">'.format(url=rs_unescaped)
+            end_a = "</a>"
         else:
-            end_a = ''
+            end_a = ""
 
-        self.write(u'{start}{rs}{end_a}</{kind}>'.format(
-            start=start_tag, rs=rs, end_a=end_a, kind=kind), indent)
+        self.write(
+            "{start}{rs}{end_a}</{kind}>".format(
+                start=start_tag, rs=rs, end_a=end_a, kind=kind
+            ),
+            indent,
+        )
 
-    def write_tr(self, line, indent=0, indent_delta=0, header=False,
-                 align=None, tags=None, nindex_levels=0):
+    def write_tr(
+        self,
+        line: Iterable,
+        indent: int = 0,
+        indent_delta: int = 0,
+        header: bool = False,
+        align: Optional[str] = None,
+        tags: Optional[Dict[int, str]] = None,
+        nindex_levels: int = 0,
+    ) -> None:
         if tags is None:
             tags = {}
 
         if align is None:
-            self.write('<tr>', indent)
+            self.write("<tr>", indent)
         else:
-            self.write('<tr style="text-align: {align};">'
-                       .format(align=align), indent)
+            self.write('<tr style="text-align: {align};">'.format(align=align), indent)
         indent += indent_delta
 
         for i, s in enumerate(line):
             val_tag = tags.get(i, None)
             if header or (self.bold_rows and i < nindex_levels):
-                self.write_th(s, indent, tags=val_tag)
+                self.write_th(s, indent=indent, header=header, tags=val_tag)
             else:
                 self.write_td(s, indent, tags=val_tag)
 
         indent -= indent_delta
-        self.write('</tr>', indent)
+        self.write("</tr>", indent)
 
-    def render(self):
+    def render(self) -> List[str]:
         self._write_table()
 
         if self.should_show_dimensions:
-            by = chr(215) if compat.PY3 else unichr(215)  # ×
-            self.write(u('<p>{rows} rows {by} {cols} columns</p>')
-                       .format(rows=len(self.frame),
-                               by=by,
-                               cols=len(self.frame.columns)))
+            by = chr(215)  # ×
+            self.write(
+                "<p>{rows} rows {by} {cols} columns</p>".format(
+                    rows=len(self.frame), by=by, cols=len(self.frame.columns)
+                )
+            )
 
         return self.elements
 
-    def _write_table(self, indent=0):
-        _classes = ['dataframe']  # Default class.
+    def write_result(self, buf: IO[str]) -> None:
+        buffer_put_lines(buf, self.render())
+
+    def _write_table(self, indent: int = 0) -> None:
+        _classes = ["dataframe"]  # Default class.
         use_mathjax = get_option("display.html.use_mathjax")
         if not use_mathjax:
-            _classes.append('tex2jax_ignore')
+            _classes.append("tex2jax_ignore")
         if self.classes is not None:
             if isinstance(self.classes, str):
                 self.classes = self.classes.split()
             if not isinstance(self.classes, (list, tuple)):
-                raise AssertionError('classes must be list or tuple, not {typ}'
-                                     .format(typ=type(self.classes)))
+                raise TypeError(
+                    "classes must be a string, list, or tuple, "
+                    "not {typ}".format(typ=type(self.classes))
+                )
             _classes.extend(self.classes)
 
         if self.table_id is None:
@@ -172,33 +226,34 @@ class HTMLFormatter(TableFormatter):
         else:
             id_section = ' id="{table_id}"'.format(table_id=self.table_id)
 
-        self.write('<table border="{border}" class="{cls}"{id_section}>'
-                   .format(border=self.border, cls=' '.join(_classes),
-                           id_section=id_section), indent)
+        self.write(
+            '<table border="{border}" class="{cls}"{id_section}>'.format(
+                border=self.border, cls=" ".join(_classes), id_section=id_section
+            ),
+            indent,
+        )
 
         if self.fmt.header or self.show_row_idx_names:
             self._write_header(indent + self.indent_delta)
 
         self._write_body(indent + self.indent_delta)
 
-        self.write('</table>', indent)
+        self.write("</table>", indent)
 
-    def _write_col_header(self, indent):
+    def _write_col_header(self, indent: int) -> None:
         truncate_h = self.fmt.truncate_h
         if isinstance(self.columns, ABCMultiIndex):
             template = 'colspan="{span:d}" halign="left"'
 
             if self.fmt.sparsify:
                 # GH3547
-                sentinel = com.sentinel_factory()
+                sentinel = lib.no_default
             else:
                 sentinel = False
-            levels = self.columns.format(sparsify=sentinel, adjoin=False,
-                                         names=False)
+            levels = self.columns.format(sparsify=sentinel, adjoin=False, names=False)
             level_lengths = get_level_lengths(levels, sentinel)
             inner_lvl = len(level_lengths) - 1
-            for lnum, (records, values) in enumerate(zip(level_lengths,
-                                                         levels)):
+            for lnum, (records, values) in enumerate(zip(level_lengths, levels)):
                 if truncate_h:
                     # modify the header lines
                     ins_col = self.fmt.tr_col_num
@@ -211,21 +266,23 @@ class HTMLFormatter(TableFormatter):
                             elif tag + span > ins_col:
                                 recs_new[tag] = span + 1
                                 if lnum == inner_lvl:
-                                    values = (values[:ins_col] + (u('...'),) +
-                                              values[ins_col:])
+                                    values = (
+                                        values[:ins_col] + ("...",) + values[ins_col:]
+                                    )
                                 else:
                                     # sparse col headers do not receive a ...
-                                    values = (values[:ins_col] +
-                                              (values[ins_col - 1], ) +
-                                              values[ins_col:])
+                                    values = (
+                                        values[:ins_col]
+                                        + (values[ins_col - 1],)
+                                        + values[ins_col:]
+                                    )
                             else:
                                 recs_new[tag] = span
                             # if ins_col lies between tags, all col headers
                             # get ...
                             if tag + span == ins_col:
                                 recs_new[ins_col] = 1
-                                values = (values[:ins_col] + (u('...'),) +
-                                          values[ins_col:])
+                                values = values[:ins_col] + ("...",) + values[ins_col:]
                         records = recs_new
                         inner_lvl = len(level_lengths) - 1
                         if lnum == inner_lvl:
@@ -239,8 +296,7 @@ class HTMLFormatter(TableFormatter):
                                 recs_new[tag] = span
                         recs_new[ins_col] = 1
                         records = recs_new
-                        values = (values[:ins_col] + [u('...')] +
-                                  values[ins_col:])
+                        values = values[:ins_col] + ["..."] + values[ins_col:]
 
                 # see gh-22579
                 # Column Offset Bug with to_html(index=False) with
@@ -248,7 +304,7 @@ class HTMLFormatter(TableFormatter):
                 # Initially fill row with blank cells before column names.
                 # TODO: Refactor to remove code duplication with code
                 # block below for standard columns index.
-                row = [''] * (self.row_levels - 1)
+                row = [""] * (self.row_levels - 1)
                 if self.fmt.index or self.show_col_idx_names:
                     # see gh-22747
                     # If to_html(index_names=False) do not show columns
@@ -259,9 +315,9 @@ class HTMLFormatter(TableFormatter):
                     # parity with DataFrameFormatter class.
                     if self.fmt.show_index_names:
                         name = self.columns.names[lnum]
-                        row.append(pprint_thing(name or ''))
+                        row.append(pprint_thing(name or ""))
                     else:
-                        row.append('')
+                        row.append("")
 
                 tags = {}
                 j = len(row)
@@ -273,8 +329,7 @@ class HTMLFormatter(TableFormatter):
                         continue
                     j += 1
                     row.append(v)
-                self.write_tr(row, indent, self.indent_delta, tags=tags,
-                              header=True)
+                self.write_tr(row, indent, self.indent_delta, tags=tags, header=True)
         else:
             # see gh-22579
             # Column misalignment also occurs for
@@ -282,7 +337,7 @@ class HTMLFormatter(TableFormatter):
             # Initially fill row with blank cells before column names.
             # TODO: Refactor to remove code duplication with code block
             # above for columns MultiIndex.
-            row = [''] * (self.row_levels - 1)
+            row = [""] * (self.row_levels - 1)
             if self.fmt.index or self.show_col_idx_names:
                 # see gh-22747
                 # If to_html(index_names=False) do not show columns
@@ -290,27 +345,27 @@ class HTMLFormatter(TableFormatter):
                 # TODO: Refactor to use _get_column_name_list from
                 # DataFrameFormatter class.
                 if self.fmt.show_index_names:
-                    row.append(self.columns.name or '')
+                    row.append(self.columns.name or "")
                 else:
-                    row.append('')
-            row.extend(self.columns)
+                    row.append("")
+            row.extend(self._get_columns_formatted_values())
             align = self.fmt.justify
 
             if truncate_h:
                 ins_col = self.row_levels + self.fmt.tr_col_num
-                row.insert(ins_col, '...')
+                row.insert(ins_col, "...")
 
-            self.write_tr(row, indent, self.indent_delta, header=True,
-                          align=align)
+            self.write_tr(row, indent, self.indent_delta, header=True, align=align)
 
-    def _write_row_header(self, indent):
+    def _write_row_header(self, indent: int) -> None:
         truncate_h = self.fmt.truncate_h
-        row = ([x if x is not None else '' for x in self.frame.index.names]
-               + [''] * (self.ncols + (1 if truncate_h else 0)))
+        row = [x if x is not None else "" for x in self.frame.index.names] + [""] * (
+            self.ncols + (1 if truncate_h else 0)
+        )
         self.write_tr(row, indent, self.indent_delta, header=True)
 
-    def _write_header(self, indent):
-        self.write('<thead>', indent)
+    def _write_header(self, indent: int) -> None:
+        self.write("<thead>", indent)
 
         if self.fmt.header:
             self._write_col_header(indent + self.indent_delta)
@@ -318,42 +373,52 @@ class HTMLFormatter(TableFormatter):
         if self.show_row_idx_names:
             self._write_row_header(indent + self.indent_delta)
 
-        self.write('</thead>', indent)
+        self.write("</thead>", indent)
 
-    def _write_body(self, indent):
-        self.write('<tbody>', indent)
-        fmt_values = {i: self.fmt._format_col(i) for i in range(self.ncols)}
+    def _get_formatted_values(self) -> Dict[int, List[str]]:
+        with option_context("display.max_colwidth", None):
+            fmt_values = {i: self.fmt._format_col(i) for i in range(self.ncols)}
+        return fmt_values
+
+    def _write_body(self, indent: int) -> None:
+        self.write("<tbody>", indent)
+        fmt_values = self._get_formatted_values()
 
         # write values
         if self.fmt.index and isinstance(self.frame.index, ABCMultiIndex):
-            self._write_hierarchical_rows(
-                fmt_values, indent + self.indent_delta)
+            self._write_hierarchical_rows(fmt_values, indent + self.indent_delta)
         else:
-            self._write_regular_rows(
-                fmt_values, indent + self.indent_delta)
+            self._write_regular_rows(fmt_values, indent + self.indent_delta)
 
-        self.write('</tbody>', indent)
+        self.write("</tbody>", indent)
 
-    def _write_regular_rows(self, fmt_values, indent):
+    def _write_regular_rows(
+        self, fmt_values: Mapping[int, List[str]], indent: int
+    ) -> None:
         truncate_h = self.fmt.truncate_h
         truncate_v = self.fmt.truncate_v
 
         nrows = len(self.fmt.tr_frame)
 
         if self.fmt.index:
-            fmt = self.fmt._get_formatter('__index__')
+            fmt = self.fmt._get_formatter("__index__")
             if fmt is not None:
                 index_values = self.fmt.tr_frame.index.map(fmt)
             else:
                 index_values = self.fmt.tr_frame.index.format()
 
-        row = []
+        row: List[str] = []
         for i in range(nrows):
 
             if truncate_v and i == (self.fmt.tr_row_num):
-                str_sep_row = ['...'] * len(row)
-                self.write_tr(str_sep_row, indent, self.indent_delta,
-                              tags=None, nindex_levels=self.row_levels)
+                str_sep_row = ["..."] * len(row)
+                self.write_tr(
+                    str_sep_row,
+                    indent,
+                    self.indent_delta,
+                    tags=None,
+                    nindex_levels=self.row_levels,
+                )
 
             row = []
             if self.fmt.index:
@@ -363,16 +428,19 @@ class HTMLFormatter(TableFormatter):
             # a standard index when the columns index is named.
             # Add blank cell before data cells.
             elif self.show_col_idx_names:
-                row.append('')
+                row.append("")
             row.extend(fmt_values[j][i] for j in range(self.ncols))
 
             if truncate_h:
                 dot_col_ix = self.fmt.tr_col_num + self.row_levels
-                row.insert(dot_col_ix, '...')
-            self.write_tr(row, indent, self.indent_delta, tags=None,
-                          nindex_levels=self.row_levels)
+                row.insert(dot_col_ix, "...")
+            self.write_tr(
+                row, indent, self.indent_delta, tags=None, nindex_levels=self.row_levels
+            )
 
-    def _write_hierarchical_rows(self, fmt_values, indent):
+    def _write_hierarchical_rows(
+        self, fmt_values: Mapping[int, List[str]], indent: int
+    ) -> None:
         template = 'rowspan="{span}" valign="top"'
 
         truncate_h = self.fmt.truncate_h
@@ -380,15 +448,13 @@ class HTMLFormatter(TableFormatter):
         frame = self.fmt.tr_frame
         nrows = len(frame)
 
-        idx_values = frame.index.format(sparsify=False, adjoin=False,
-                                        names=False)
-        idx_values = lzip(*idx_values)
+        idx_values = frame.index.format(sparsify=False, adjoin=False, names=False)
+        idx_values = list(zip(*idx_values))
 
         if self.fmt.sparsify:
             # GH3547
-            sentinel = com.sentinel_factory()
-            levels = frame.index.format(sparsify=sentinel, adjoin=False,
-                                        names=False)
+            sentinel = lib.no_default
+            levels = frame.index.format(sparsify=sentinel, adjoin=False, names=False)
 
             level_lengths = get_level_lengths(levels, sentinel)
             inner_lvl = len(level_lengths) - 1
@@ -396,6 +462,8 @@ class HTMLFormatter(TableFormatter):
                 # Insert ... row and adjust idx_values and
                 # level_lengths to take this into account.
                 ins_row = self.fmt.tr_row_num
+                # cast here since if truncate_v is True, self.fmt.tr_row_num is not None
+                ins_row = cast(int, ins_row)
                 inserted = False
                 for lnum, records in enumerate(level_lengths):
                     rec_new = {}
@@ -408,12 +476,12 @@ class HTMLFormatter(TableFormatter):
                             # GH 14882 - Make sure insertion done once
                             if not inserted:
                                 dot_row = list(idx_values[ins_row - 1])
-                                dot_row[-1] = u('...')
+                                dot_row[-1] = "..."
                                 idx_values.insert(ins_row, tuple(dot_row))
                                 inserted = True
                             else:
                                 dot_row = list(idx_values[ins_row])
-                                dot_row[inner_lvl - lnum] = u('...')
+                                dot_row[inner_lvl - lnum] = "..."
                                 idx_values[ins_row] = tuple(dot_row)
                         else:
                             rec_new[tag] = span
@@ -422,19 +490,20 @@ class HTMLFormatter(TableFormatter):
                         if tag + span == ins_row:
                             rec_new[ins_row] = 1
                             if lnum == 0:
-                                idx_values.insert(ins_row, tuple(
-                                    [u('...')] * len(level_lengths)))
+                                idx_values.insert(
+                                    ins_row, tuple(["..."] * len(level_lengths))
+                                )
 
                             # GH 14882 - Place ... in correct level
                             elif inserted:
                                 dot_row = list(idx_values[ins_row])
-                                dot_row[inner_lvl - lnum] = u('...')
+                                dot_row[inner_lvl - lnum] = "..."
                                 idx_values[ins_row] = tuple(dot_row)
                     level_lengths[lnum] = rec_new
 
                 level_lengths[inner_lvl][ins_row] = 1
                 for ix_col in range(len(fmt_values)):
-                    fmt_values[ix_col].insert(ins_row, '...')
+                    fmt_values[ix_col].insert(ins_row, "...")
                 nrows += 1
 
             for i in range(nrows):
@@ -456,27 +525,44 @@ class HTMLFormatter(TableFormatter):
 
                 row.extend(fmt_values[j][i] for j in range(self.ncols))
                 if truncate_h:
-                    row.insert(self.row_levels - sparse_offset +
-                               self.fmt.tr_col_num, '...')
-                self.write_tr(row, indent, self.indent_delta, tags=tags,
-                              nindex_levels=len(levels) - sparse_offset)
+                    row.insert(
+                        self.row_levels - sparse_offset + self.fmt.tr_col_num, "..."
+                    )
+                self.write_tr(
+                    row,
+                    indent,
+                    self.indent_delta,
+                    tags=tags,
+                    nindex_levels=len(levels) - sparse_offset,
+                )
         else:
             row = []
             for i in range(len(frame)):
                 if truncate_v and i == (self.fmt.tr_row_num):
-                    str_sep_row = ['...'] * len(row)
-                    self.write_tr(str_sep_row, indent, self.indent_delta,
-                                  tags=None, nindex_levels=self.row_levels)
+                    str_sep_row = ["..."] * len(row)
+                    self.write_tr(
+                        str_sep_row,
+                        indent,
+                        self.indent_delta,
+                        tags=None,
+                        nindex_levels=self.row_levels,
+                    )
 
-                idx_values = list(zip(*frame.index.format(
-                    sparsify=False, adjoin=False, names=False)))
+                idx_values = list(
+                    zip(*frame.index.format(sparsify=False, adjoin=False, names=False))
+                )
                 row = []
                 row.extend(idx_values[i])
                 row.extend(fmt_values[j][i] for j in range(self.ncols))
                 if truncate_h:
-                    row.insert(self.row_levels + self.fmt.tr_col_num, '...')
-                self.write_tr(row, indent, self.indent_delta, tags=None,
-                              nindex_levels=frame.index.nlevels)
+                    row.insert(self.row_levels + self.fmt.tr_col_num, "...")
+                self.write_tr(
+                    row,
+                    indent,
+                    self.indent_delta,
+                    tags=None,
+                    nindex_levels=frame.index.nlevels,
+                )
 
 
 class NotebookFormatter(HTMLFormatter):
@@ -486,7 +572,13 @@ class NotebookFormatter(HTMLFormatter):
     DataFrame._repr_html_() and DataFrame.to_html(notebook=True)
     """
 
-    def write_style(self):
+    def _get_formatted_values(self) -> Dict[int, List[str]]:
+        return {i: self.fmt._format_col(i) for i in range(self.ncols)}
+
+    def _get_columns_formatted_values(self) -> List[str]:
+        return self.columns.format()
+
+    def write_style(self) -> None:
         # We use the "scoped" attribute here so that the desired
         # style properties for the data frame are not then applied
         # throughout the entire notebook.
@@ -498,34 +590,25 @@ class NotebookFormatter(HTMLFormatter):
                 .dataframe %s {
                     %s: %s;
                 }"""
-        element_props = [('tbody tr th:only-of-type',
-                          'vertical-align',
-                          'middle'),
-                         ('tbody tr th',
-                          'vertical-align',
-                          'top')]
+        element_props = [
+            ("tbody tr th:only-of-type", "vertical-align", "middle"),
+            ("tbody tr th", "vertical-align", "top"),
+        ]
         if isinstance(self.columns, ABCMultiIndex):
-            element_props.append(('thead tr th',
-                                  'text-align',
-                                  'left'))
+            element_props.append(("thead tr th", "text-align", "left"))
             if self.show_row_idx_names:
-                element_props.append(('thead tr:last-of-type th',
-                                      'text-align',
-                                      'right'))
+                element_props.append(
+                    ("thead tr:last-of-type th", "text-align", "right")
+                )
         else:
-            element_props.append(('thead th',
-                                  'text-align',
-                                  'right'))
-        template_mid = '\n\n'.join(map(lambda t: template_select % t,
-                                       element_props))
-        template = dedent('\n'.join((template_first,
-                                     template_mid,
-                                     template_last)))
+            element_props.append(("thead th", "text-align", "right"))
+        template_mid = "\n\n".join(map(lambda t: template_select % t, element_props))
+        template = dedent("\n".join((template_first, template_mid, template_last)))
         self.write(template)
 
-    def render(self):
-        self.write('<div>')
+    def render(self) -> List[str]:
+        self.write("<div>")
         self.write_style()
-        super(NotebookFormatter, self).render()
-        self.write('</div>')
+        super().render()
+        self.write("</div>")
         return self.elements
